<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.21
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2022 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base\Assignments;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

use FPFramework\Base\Assignment;

class Menu extends Assignment 
{
	protected $itemID = null;

	public function __construct($options, $factory = null)
	{
		parent::__construct($options, $factory);

		if (!is_array($this->selection))
		{
			$this->selection = $this->splitKeywords($this->selection);
		}

        $current_object_id = \FPFramework\Helpers\WPHelper::getPageID();
		
		$this->itemID = \FPFramework\Helpers\MenuHelper::getMenuIdByPostId($current_object_id);
	}
	
	/**
	 *  Pass check for menu items
	 *
	 *  @return  bool
	 */
	public function pass()
	{
        $includeChildren = isset($this->params->inc_children) ? $this->params->inc_children : false;
		
        // Don't pass if selection is empty
        if (empty($this->selection))
        {
            return false;
        }

		// return true if menu is in selection and "include child items" is not set to only
		if (in_array($this->itemID, $this->selection))
		{
			return ($includeChildren != 2);
		}
        
		// Let's discover child items. 
		// Obviously if the option is disabled return false.
		if (!$includeChildren)
		{
			return false;
        }
        
        $parent_ids = $this->getParentIds($this->itemID);
		$parent_ids = array_diff($parent_ids, array('1'));

		foreach ($parent_ids as $id)
		{
			if (!in_array($id, $this->selection))
			{
				continue;
			}

			return true;
		}

        return false;
    }
    
	/**
	 *  Returns all parent rows
	 *
	 *  @param   integer  $id      Row primary key
	 *  @param   string   $table   Table name
	 *  @param   string   $parent  Parent column name
	 *  @param   string   $child   Child column name
	 *
	 *  @return  array             Array with IDs
	 */
    public function getParentIds($id = 0, $table = 'postmeta', $parent = 'meta_value', $child = '_menu_item_menu_item_parent')
    {
		// cache key
		$hash = md5('menuGetParentIds_' . $id . '_' . $table . '_' . $parent . '_' . $child);

		// check cache
		if ($parent_ids = wp_cache_get($hash))
		{
			return $parent_ids;
        }
        
		global $wpdb;
		
        $parent_ids = array();
        
        $table = $wpdb->prefix . $table;
        
		while ($id)
		{
			$args = [
				esc_sql((int) $id),
				esc_sql($child)
			];
			
			$id = $wpdb->get_row($wpdb->prepare("SELECT t." . esc_sql($parent) . " as parent FROM " . esc_sql($table) . " as t WHERE t.post_id = %d AND t.meta_key = %s", $args));

			if (!isset($id->parent))
			{
				break;
			}
			$id = $id->parent;
            
			// Break if no parent is found or parent already found before for some reason
			if (!$id || in_array($id, $parent_ids))
			{
				break;
            }
            
            $parent_ids[] = $id;
        }

		// set cache
		wp_cache_set($hash, $parent_ids);

		return $parent_ids;
    }

	/**
     *  Returns the assignment's value
     * 
     *  @return integer Menu ID
     */
	public function value()
	{
		return $this->itemID;
	}
}
