<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.21
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2022 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base\Widgets;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

/**
 * Countdown
 */
class Countdown extends Widget
{
	/**
	 * Widget default options
	 *
	 * @var array
	 */
	protected $widget_options = [
		/**
		 * The Countdown type:
		 * 
		 * - static: Counts down to a specific date and time. Universal deadline for all visitors.
		 * - dynamic: Set-and-forget solution. The countdown starts when your visitor sees the offer.
		 */
		'countdown_type' => 'static',

		// The Static Countdown Date
		'value' => '',

		/**
		 * The timezone that will be used.
		 * 
		 * - server - Use server's timezone
		 * - client - Use client's timezone
		 */
		'timezone' => 'server',

		// Dynamic Days
		'dynamic_days' => 0,

		// Dynamic Hours
		'dynamic_hours' => 0,

		// Dynamic Minutes
		'dynamic_minutes' => 0,

		// Dynamic Seconds
		'dynamic_seconds' => 0,
		
		/**
		 * The countdown format.
		 * 
		 * Available tags:
		 * {years}
		 * {months}
		 * {days}
		 * {hours}
		 * {minutes}
		 * {seconds}
		 */
		'format' => '{days} days, {hours} hours, {minutes} minutes and {seconds} seconds',

		/**
		 * The countdown theme.
		 * 
		 * Available themes:
		 * theme1
		 * theme2
		 * theme3
		 */
		'theme' => 'theme1',

		/**
		 * Set the action once countdown finishes.
		 * 
		 * Available values:
		 * keep 	- Keep the countdown visible
		 * hide 	- Hide the countdown
		 * message	- Show a message
		 * redirect	- Redirect to a URL
		 */
		'countdown_action' => 'keep',

		/**
		 * The message appearing after the countdown has finished.
		 * 
		 * Requires `countdown_action` to be set to `message`
		 * 
		 * Example: Countdown finished.
		 */
		'finish_text' => '',

		/**
		 * The redirect URL once the countdown expires.
		 * 
		 * Requires `countdown_action` to be set to `redirect`
		 */
		'redirect_url' => '',

		/**
		 * Block Settings
		 */
		// Alignment
		'align' => '',

		// Padding
		'padding' => [
			'desktop' => 0,
			'tablet' => 0,
			'mobile' => 0
		],

		// Margin
		'margin' => [
			'desktop' => 0,
			'tablet' => 0,
			'mobile' => 0
		],

		// Gap
		'gap' => [
			'desktop' => 20,
			'tablet' => 20,
			'mobile' => 20
		],
		
		// Background Color
		'background_color' => '',

		/**
		 * Unit Display Settings
		 */
		// Whether to display Days
		'days' => true,

		// Days Label
		'days_label' => 'Days',
		
		// Whether to display Hours
		'hours' => true,

		// Hours Label
		'hours_label' => 'Hrs',
		
		// Whether to display Minutes
		'minutes' => true,

		// Minutes Label
		'minutes_label' => 'Mins',
		
		// Whether to display Seconds
		'seconds' => true,

		// Seconds Label
		'seconds_label' => 'Secs',

		/**
		 * Unit Item Settings
		 */
		// The size (width, height) of the unit item in pixels
		'item_size' => [],
		
		// The unit item border width
		'item_border_width' => '',

		// The unit item border style
		'item_border_style' => '',

		// The unit item border color
		'item_border_color' => '',

		// The unit item border radius
		'item_border_radius' => '',

		// Item Background Color
		'item_background_color' => '',

		/**
		 * Unit Digits Settings
		 */
		// Digits Font Size
		'digits_font_size' => [
			'desktop' => 25,
			'tablet' => 25,
			'mobile' => 25
		],

		// Digits Font Weight
		'digits_font_weight' => '400',

		// Digit Min Width
		'digit_min_width' => 16,

		// The digits padding
		'digits_padding' => '',

		// The digits border radius
		'digit_border_radius' => [],

		// Digit Item Background Color. This applies for each of the 2 digits on a unit.
		'digit_background_color' => '',

		// Digit Item Text Color
		'digit_text_color' => '',

		/**
		 * Unit Label Settings
		 */
		// Label Font Size
		'label_font_size' => [
			'desktop' => 13,
			'tablet' => 13,
			'mobile' => 13
		],

		// Label Font Weight
		'label_font_weight' => '400',

		// Unit Label Margin Top. The spacing between the unit and its label.
		'unit_label_margin_top' => '',

		// Unit Label Color
		'unit_label_text_color' => '',

		// Extra attributes added to the widget
		'atts' => '',

		// Custom CSS printed after the widget assets
		'custom_css' => ''
	];

	/**
	 * Class constructor
	 *
	 * @param array $options
	 */
	public function __construct($options = [])
	{
		parent::__construct($options);

		$this->prepare();
	}

	/**
	 * Prepares the countdown.
	 * 
	 * @return  void
	 */
	private function prepare()
	{
		$this->setCSSVars();
		$this->setResponsiveCSS();

		$this->options['css_class'] .= ' ' . $this->options['theme'] . ' ' . $this->options['align'];

		if (!empty($this->options['value']) && $this->options['value'] !== '0000-00-00 00:00:00')
		{
			if ($this->options['countdown_type'] === 'static' && $this->options['timezone'] === 'server')
			{
				// Convert given date time to UTC
				$this->options['value'] = get_gmt_from_date($this->options['value'], 'c');
				
				// Get timezone
				$tz = new \DateTimeZone(wp_timezone()->getName());

				// Apply server timezone
				$this->options['value'] = (new \DateTime($this->options['value']))->setTimezone($tz)->format('c');
			}
		}

		// Set countdown payload
		$payload = [
			'data-countdown-type="' . esc_attr($this->options['countdown_type']) . '"',
			'data-value="' . esc_attr($this->options['value']) . '"',
			'data-timezone="' . esc_attr($this->options['timezone']) . '"',
			'data-dynamic-days="' . esc_attr($this->options['dynamic_days']) . '"',
			'data-dynamic-hours="' . esc_attr($this->options['dynamic_hours']) . '"',
			'data-dynamic-minutes="' . esc_attr($this->options['dynamic_minutes']) . '"',
			'data-dynamic-seconds="' . esc_attr($this->options['dynamic_seconds']) . '"',
			'data-finish-text="' . esc_attr(htmlspecialchars($this->options['finish_text'])) . '"',
			'data-redirect-url="' . esc_attr($this->options['redirect_url']) . '"',
			'data-theme="' . esc_attr($this->options['theme']) . '"',
			'data-format="' . esc_attr(htmlspecialchars($this->options['format'])) . '"',
			'data-countdown-action="' . esc_attr($this->options['countdown_action']) . '"',
			'data-days="' . esc_attr(var_export($this->options['days'], true)) . '"',
			'data-days-label="' . esc_attr($this->options['days_label']) . '"',
			'data-hours="' . esc_attr(var_export($this->options['hours'], true)) . '"',
			'data-hours-label="' . esc_attr($this->options['hours_label']) . '"',
			'data-minutes="' . esc_attr(var_export($this->options['minutes'], true)) . '"',
			'data-minutes-label="' . esc_attr($this->options['minutes_label']) . '"',
			'data-seconds="' . esc_attr(var_export($this->options['seconds'], true)) . '"',
			'data-seconds-label="' . esc_attr($this->options['seconds_label']) . '"'
		];
		$this->options['atts'] = implode(' ', $payload);
	}

	/**
	 * Set widget CSS vars
	 * 
	 * @return  mixed
	 */
	private function setCSSVars()
	{
		if (!$this->options['load_css_vars'])
		{
			return;
		}

		$atts = [];

		if (!empty($this->options['background_color']))
		{
			$atts['background-color'] = $this->options['background_color'];
		}

		if (!empty($this->options['item_background_color']))
		{
			$atts['item-background-color'] = $this->options['item_background_color'];
		}

		if (!empty($this->options['unit_label_text_color']))
		{
			$atts['unit-label-text-color'] = $this->options['unit_label_text_color'];
		}

		if (!empty($this->options['digit_background_color']))
		{
			$atts['digit-background-color'] = $this->options['digit_background_color'];
		}

		if (!empty($this->options['digit_text_color']))
		{
			$atts['digit-text-color'] = $this->options['digit_text_color'];
		}

		if (!empty($this->options['unit_label_margin_top']))
		{
			$atts['unit-label-margin-top'] = $this->options['unit_label_margin_top'] . 'px';
		}

		if (!empty($this->options['digit_min_width']))
		{
			$atts['digit-min-width'] = $this->options['digit_min_width'] . 'px';
		}

		if (!empty($this->options['digits_font_weight']))
		{
			$atts['digits-font-weight'] = $this->options['digits_font_weight'];
		}

		if (!empty($this->options['label_font_weight']))
		{
			$atts['label-font-weight'] = $this->options['label_font_weight'];
		}

		if (!empty($this->options['digit_border_radius']))
		{
			// remove linked state
			unset($this->options['digit_border_radius']['linked']);

			// suffix values
			$suffixed_border_radius = array_map(
				function($item) {
					return $item . 'px';
				},
				$this->options['digit_border_radius']
			);
			
			$atts['digit-border-radius'] = implode(' ', $suffixed_border_radius);
		}

		if (!empty($this->options['item_border_width']) && !empty($this->options['item_border_style']) && !empty($this->options['item_border_color']))
		{
			$atts['item-border'] = $this->options['item_border_width'] . 'px ' . $this->options['item_border_style'] . ' ' . $this->options['item_border_color'];
		}

		if (empty($atts))
		{
			return;
		}

		if (!$css = \FPFramework\Helpers\CSS::cssVarsToString($atts, '.fpf-countdown.' . $this->options['id']))
		{
			return;
		}

		$this->options['custom_css'] = $css;
	}

	/**
	 * Sets the CSS for the responsive settings.
	 * 
	 * @return  void
	 */
	private function setResponsiveCSS()
	{
		$initial_breakpoints = [
			'desktop' => [],
			'tablet' => [],
			'mobile' => []
		];
		$responsive_css = $initial_breakpoints;
		
		// Add block padding
		if ($padding = \FPFramework\Helpers\Controls\Spacing::getResponsiveSpacingControlValue($this->options['padding'], 'padding', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $padding);
		}

		// Add block margin
		if ($margin = \FPFramework\Helpers\Controls\Spacing::getResponsiveSpacingControlValue($this->options['margin'], 'margin', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $margin);
		}
		
		// Add gap
		if ($gap = \FPFramework\Helpers\Controls\Responsive::prepareResponsiveControlValue($this->options['gap'], '--gap', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $gap);
		}

		// Add Item Size
		if ($item_size = \FPFramework\Helpers\Controls\Responsive::prepareResponsiveControlValue($this->options['item_size'], '--item-size', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $item_size);
		}

		// Add Digits Font Size
		if ($digits_font_size = \FPFramework\Helpers\Controls\Responsive::prepareResponsiveControlValue($this->options['digits_font_size'], '--digits-font-size', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $digits_font_size);
		}

		// Add Label Font Size
		if ($label_font_size = \FPFramework\Helpers\Controls\Responsive::prepareResponsiveControlValue($this->options['label_font_size'], '--label-font-size', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $label_font_size);
		}

		if ($css = \FPFramework\Helpers\Responsive::renderResponsiveCSS($responsive_css, '.fpf-countdown.' . $this->options['id']))
		{
			$this->options['custom_css'] .= $css;
		}

		// Add item border radius
		if ($itemBorderRadius = \FPFramework\Helpers\Controls\Spacing::getResponsiveSpacingControlValue($this->options['item_border_radius'], 'border-radius', 'px'))
		{
			$responsive_css = $initial_breakpoints;
			$responsive_css = array_merge_recursive($responsive_css, $itemBorderRadius);

			if ($css = \FPFramework\Helpers\Responsive::renderResponsiveCSS($responsive_css, '.fpf-countdown.' . $this->options['id'] . ' .countdown-item'))
			{
				$this->options['custom_css'] .= $css;
			}
		}

		// Add Digits Padding
		if ($digitsPadding = \FPFramework\Helpers\Controls\Spacing::getResponsiveSpacingControlValue($this->options['digits_padding'], 'padding', 'px'))
		{
			$responsive_css = $initial_breakpoints;
			$responsive_css = array_merge_recursive($responsive_css, $digitsPadding);
			
			if ($css = \FPFramework\Helpers\Responsive::renderResponsiveCSS($responsive_css, '.fpf-countdown.' . $this->options['id'] . ' .countdown-item .countdown-digit .digit-number'))
			{
				$this->options['custom_css'] .= $css;
			}
		}
	}
}