<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.21
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2022 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base\Widgets;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

/**
 *  Notification
 */
class Notification extends Widget
{
	/**
	 * Widget default options
	 *
	 * @var array
	 */
	protected $widget_options = [
		/**
		 * The type.
		 * 
		 * Available values:
		 * - information
		 * - success
		 * - error
		 * - warning
		 */
		'type' => 'information',

		/**
		 * The theme.
		 * 
		 * Available values:
		 * 
		 * - theme1
		 * - theme2
		 * - theme3
		 */
		'theme' => 'theme1',

		/**
		 * Whether to display the title.
		 * 
		 * If null, the theme provides the value.
		 */
		'enable_title' => null,

		// The title
		'title' => '',

		// The message
		'message' => '',

		/**
		 * The skin.
		 * 
		 * Available values:
		 * 
		 * - light
		 * - color
		 * - dark
		 * - custom (Allows to select custom text and background color)
		 */
		'skin' => 'light',

		/**
		 * The border radius.
		 */
		'border_radius' => [],

		/**
		 * Whether to display the close button or not.
		 * 
		 * If null, the theme provides the value.
		 */
		'show_close_button' => null,

		// The text color
		'text_color' => null,

		// The background color
		'background_color' => null,

		// Margin
		'margin' => [
			'desktop' => 0,
			'tablet' => 0,
			'mobile' => 0
		],

		// Extra attributes added to the widget
		'atts' => '',

		// Custom CSS printed after the widget assets
		'custom_css' => ''
	];

	/**
	 * Class constructor
	 *
	 * @param array $options
	 */
	public function __construct($options = [])
	{
		parent::__construct($options);

		$this->options['css_class'] .= ' ' . $this->options['theme'] . ' type-' . $this->options['type'] . ' ' . $this->options['skin'];

		// On readonly, do not allow interaction (closing of notification via close button)
		if ($this->options['readonly'])
		{
			$this->options['css_class'] .= ' readonly';
		}

		if (is_null($this->options['enable_title']))
		{
			if (in_array($this->options['theme'], ['theme2', 'theme3']))
			{
				$this->options['enable_title'] = true;
			}
		}

		$this->setCSSVars();
		$this->setCustomCSS();
	}

	/**
	 * Sets CSS variables.
	 * 
	 * @return  void
	 */
	private function setCSSVars()
	{
		if (!$this->options['load_css_vars'])
		{
			return;
		}
		
		$css_vars = [];

		// Custom colors apply only to custom skin
		if ($this->options['skin'] === 'custom')
		{
			if (!empty($this->options['text_color']))
			{
				$css_vars['text-color'] = $this->options['text_color'];
			}

			if (!empty($this->options['background_color']))
			{
				$css_vars['background-color'] = $this->options['background_color'];
			}
		}

		if (!empty($this->options['border_radius']))
		{
			// remove linked state
			unset($this->options['border_radius']['linked']);

			// suffix values
			$suffixed_border_radius = array_map(
				function($item) {
					return $item . 'px';
				},
				$this->options['border_radius']
			);
			
			$css_vars['border-radius'] = implode(' ', $suffixed_border_radius);
		}

		if (empty($css_vars))
		{
			return;
		}

		if (!$css = \FPFramework\Helpers\CSS::cssVarsToString($css_vars, '.fpf-notification.' . $this->options['id']))
		{
			return;
		}

		$this->options['custom_css'] = $css;
	}

	/**
	 * Sets the CSS for the responsive settings.
	 * 
	 * @return  void
	 */
	private function setCustomCSS()
	{
		$responsive_css = [
			'desktop' => [],
			'tablet' => [],
			'mobile' => []
		];
		
		// Add margin
		if ($margin = \FPFramework\Helpers\Controls\Spacing::getResponsiveSpacingControlValue($this->options['margin'], 'margin', 'px'))
		{
			$responsive_css = array_merge_recursive($responsive_css, $margin);
		}

		if (!$css = \FPFramework\Helpers\Responsive::renderResponsiveCSS($responsive_css, '.fpf-notification.' . $this->options['id']))
		{
			return;
		}

		$this->options['custom_css'] .= $css;
	}
}