<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.21
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2022 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Helpers\Controls;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

class Spacing
{
    /**
     * Default Spacing Control Posiitons.
     * 
     * @var  array
     */
    public static $spacing_positions = ['top', 'right', 'bottom', 'left'];
    
    /**
     * Returns the CSS of the spacing control.
     * 
     * @param   array   $value
     * @param   string  $prefix
     * @param   string  $breakpoint
     * @param   string  $unit
     * 
     * @return  string
     */
    public static function getResponsiveSpacingControlValue($value, $prefix = '', $unit = '', $breakpoint = '')
    {
        if (!is_array($value))
        {
            return;
        }

        // Return the value for a specific breakpoint
        if (!empty($breakpoint) && is_string($breakpoint))
        {
            if (!isset($value[$breakpoint]))
            {
                return;
            }

            return $prefix . ': ' . self::getSpacingValue($value[$breakpoint], $unit) . ';';
        }
        
        // Return the value for all breakpoints
        $css = [];

        // remove linked property
        unset($value['linked']);

        foreach ($value as $_breakpoint => $values)
        {
            if (!$value = self::getSpacingValue($values, $unit))
            {
                continue;
            }

            $css[$_breakpoint] = $prefix . ': ' . $value . ';';
        }

        return $css;
    }

    /**
     * Returns the value of a spacing control (margin, padding).
     * 
     * @param   array   $value
     * @param   string  $unit
     * 
     * @return  string
     */
    public static function getSpacingValue($value, $unit = '')
    {
        if (!$value || !is_array($value))
        {
            return;
        }
        
        $return = [];

        foreach (self::$spacing_positions as $pos)
        {
            $return[$pos] = isset($value[$pos]) && $value[$pos] !== '' ? $value[$pos] : 0;
        }

        // If all spacing values are empty, return
        if (self::isEmpty($return))
        {
            return;
        }

        /**
         * All values are duplicates, return only 1 number with their unit.
         * 
         * if given [5, 5, 5, 5] to print the margin in pixels, do not return `margin: 5px 5px 5px 5px`.
         * Rather return `margin: 5px`
         */
        if (count(array_unique($return)) === 1)
        {
            return reset($return) . $unit;
        }

        // add unit suffix
        $return = preg_filter('/$/', $unit, $return);

        return implode(' ', $return);
    }

    /**
     * Checks whether the spacing value is empty.
     * 
     * @param   array    $value
     * 
     * @return  boolean
     */
    public static function isEmpty($value)
    {
        if (!is_array($value))
        {
            return false;
        }
        
        foreach (self::$spacing_positions as $pos)
        {
            if (!isset($value[$pos]) || empty($value[$pos]))
            {
                continue;
            }

            return false;
        }

        return true;
    }
}